const int redLedPin = 13;  // Analog input pin that the potentiometer is attached to
const int greenLedPin = 10; // Analog output pin that the LED is attached to
const int amberLedPin = 11;  // Analog input pin that the potentiometer is attached to
const int microphonePin = A0; // Analog output pin that the LED is attached to
const int potentiometerLeftPin = A2;  // Analog input pin that the potentiometer is attached to
const int potentiometerRightPin = A1; // Analog output pin that the LED is attached to

const int sampleWindow = 50; // Sample window width in mS (50 mS = 20Hz)
unsigned int sample, sensitivity, duration;

// Define the number of samples to keep track of. The higher the number, the
// more the readings will be smoothed, but the slower the output will respond to
// the input. Using a constant rather than a normal variable lets us use this
// value to determine the size of the readings array.
const int numReadings = 5;

int readings[numReadings];      // the readings from the analog input
int readIndex = 0;              // the index of the current reading
int total = 0;                  // the running total
int average = 0;                // the average

void setup() {
  pinMode(redLedPin, OUTPUT);
  pinMode(greenLedPin, OUTPUT);
  pinMode(amberLedPin, OUTPUT);

  Serial.begin(9600);

  // initialize all the readings to 0:
  for (int thisReading = 0; thisReading < numReadings; thisReading++) {
    readings[thisReading] = 0;
  }
}

void loop() {
  
  int average = averageVolume();
  int delayLed = readDelay();

  if (average > 25) {
    digitalWrite(redLedPin, HIGH);
    digitalWrite(amberLedPin, LOW);
    digitalWrite(greenLedPin, LOW); 
    delay(delayLed); 
  } else if (average > 20 && average <= 25) {
    digitalWrite(amberLedPin, HIGH);
    digitalWrite(greenLedPin, LOW);
    digitalWrite(redLedPin, LOW); 
    delay(delayLed); 
  } else {
    digitalWrite(greenLedPin, HIGH);
    digitalWrite(redLedPin, LOW);  
    digitalWrite(amberLedPin, LOW);    
  }
}

int readDelay() {
   int value = map(analogRead(potentiometerRightPin), 1024, 0, 0, 5000);  
   // Serial.println(value);
   return value;
}

int averageVolume() {
  // subtract the last reading:
  total = total - readings[readIndex];
  // read from the sensor:
  readings[readIndex] = measureVolume();
  // add the reading to the total:
  total = total + readings[readIndex];
  // advance to the next position in the array:
  readIndex = readIndex + 1;

  // if we're at the end of the array...
  if (readIndex >= numReadings) {
    // ...wrap around to the beginning:
    readIndex = 0;
  }

  // calculate the average:
  average = total / numReadings;
  // send it to the computer as ASCII digits
  // Serial.println(average);
  return average;
}

int measureVolume() {
   unsigned long startMillis= millis();  // Start of sample window
   unsigned int peakToPeak = 0;   // peak-to-peak level
 
   unsigned int signalMax = 0;
   unsigned int signalMin = 1024;
 
   // collect data for 50 mS
   while (millis() - startMillis < sampleWindow)
   {
      sample = analogRead(microphonePin);
      if (sample < 1024)  // toss out spurious readings
      {
         if (sample > signalMax)
         {
            signalMax = sample;  // save just the max levels
         }
         else if (sample < signalMin)
         {
            signalMin = sample;  // save just the min levels
         }
      }
   }
   peakToPeak = signalMax - signalMin;  // max - min = peak-peak amplitude
   double volts = (peakToPeak * 5.0) / 1024;  // convert to volts

   int sensitivity  = map(analogRead(potentiometerLeftPin), 1024, 0, 0, 100);
   int volume  = map(peakToPeak, 0, 1023, 0, sensitivity);
 
   Serial.println(volume);
   return volume;
}
